<?php
declare(strict_types=1);

namespace Catequil\Webshop;

use Laminas\Hydrator\ClassMethodsHydrator;
use Laminas\ServiceManager\AbstractFactory\ReflectionBasedAbstractFactory;

/**
 * Class ConfigProvider
 *
 * @package Catequil\Webshop
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'dependencies' => $this->getDependencies(),
            'filters' => $this->getFilters(),
            'hydrators' => $this->getHydrators(),
            'orm' => $this->getOrmConfig(),
            'routes' => $this->getRoutes(),
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                Service\WebshopServiceInterface::class => Service\WebshopService::class,
            ],
        ];
    }

    public function getFilters(): array
    {
        return [
            'factories' => [
                Filter\WebshopFilter::class => ReflectionBasedAbstractFactory::class,
            ],
        ];
    }

    public function getHydrators(): array
    {
        return [
            'aliases' => [
                Model\Webshop::class => Hydrator\WebshopHydrator::class,
                Model\WebshopOrder::class => Hydrator\WebshopOrderHydrator::class,
            ],
            'invokables' => [
                Hydrator\WebshopHydrator::class,
                Hydrator\WebshopOrderHydrator::class,
            ],
        ];
    }

    public function getOrmConfig(): array
    {
        return [
            'implementations' => [
                Model\WebshopInterface::class => Model\Webshop::class,
                Model\WebshopOrderInterface::class => Model\WebshopOrder::class,
            ],
            'entity_paths' => [
                __NAMESPACE__ => __DIR__ . '/Model',
            ],
            'event_subscribers' => [
                Service\WebshopEventSubscriber::class,
            ],
        ];
    }

    public function getRoutes(): array
    {
        return [[
            'name' => 'webshop.order.show',
            'path' => '/api/webshop/:domain/order/:id',
            'allowed_methods' => ['GET'],
            'middleware' => Handler\Order\ShowOrder::class,
        ], [
            'name' => 'webshop.order.update',
            'path' => '/api/webshop/:domain/order/:id/update',
            'allowed_methods' => ['GET'],
            'middleware' => Handler\Order\UpdateOrder::class,
        ]];
    }
}