<?php
declare(strict_types=1);

namespace Catequil\Webshop\Handler\Order;

use Catequil\Webshop\Service\WebshopServiceInterface;
use Laminas\Diactoros\Response\JsonResponse;
use Laminas\Hydrator\HydratorInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use function intval;

/**
 * Class UpdateOrder
 *
 * @package Catequil\Webshop\Handler\Order
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class UpdateOrder implements RequestHandlerInterface
{
    private WebshopServiceInterface $webshops;
    private HydratorInterface $hydrator;

    /**
     * @param WebshopServiceInterface $webshops
     * @param HydratorInterface $hydrator
     */
    public function __construct(WebshopServiceInterface $webshops, HydratorInterface $hydrator)
    {
        $this->webshops = $webshops;
        $this->hydrator = $hydrator;
    }

    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        $order = $this->webshops->orders()->update(
            $request->getAttribute('domain'),
            intval($request->getAttribute('id'))
        );

        return new JsonResponse(
            $this->hydrator->extract($order)
        );
    }
}