<?php
declare(strict_types=1);

namespace Catequil\Webshop\Hydrator;

use Catequil\Core\Hydrator\DefaultHydrator;
use Catequil\Webshop\Model\Webshop;
use Catequil\Webshop\Model\WebshopInterface;
use Laminas\Hydrator\Filter\FilterComposite;
use Laminas\Hydrator\Filter\MethodMatchFilter;
use Laminas\Hydrator\Strategy\ClosureStrategy;
use Laminas\Hydrator\Strategy\DateTimeFormatterStrategy;
use Laminas\Hydrator\Strategy\HydratorStrategy;

/**
 * Class WebshopOrderHydrator
 *
 * @package Catequil\Webshop\Hydrator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class WebshopOrderHydrator extends DefaultHydrator
{
    public function __construct()
    {
        parent::__construct();

        $this->addStrategy('webshop', new HydratorStrategy(new WebshopHydrator(), Webshop::class));
        $this->addStrategy('updatedAt', new DateTimeFormatterStrategy('Y-m-d H:i:s'));

        $filter = new FilterComposite();
        $filter->addFilter('details', new MethodMatchFilter('getDetails'), FilterComposite::CONDITION_AND);
        $filter->addFilter('items', new MethodMatchFilter('getItems'), FilterComposite::CONDITION_AND);

        $this->addFilter('excludes', $filter, FilterComposite::CONDITION_AND);
    }
}