<?php
declare(strict_types=1);

namespace Catequil\Webshop\Repository;

use Catequil\Webshop\Model\Webshop;
use Catequil\Webshop\Model\WebshopInterface;
use Catequil\Webshop\Model\WebshopOrder;
use Catequil\Webshop\Model\WebshopOrderInterface;
use Doctrine\ORM\EntityRepository;
use InvalidArgumentException;
use function is_string;
use function sprintf;

/**
 * Class WebshopOrderRepository
 *
 * @package Catequil\Webshop\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class WebshopOrderRepository extends EntityRepository
{
    public function find($id, $lockMode = null, $lockVersion = null)
    {
        [$webshop, $id] = $id;

        return $this->findOneBy([
            'webshop' => $this->getWebshop($webshop),
            'id' => $id,
        ]);
    }

    public function findOrCreate(int|string|WebshopInterface $webshop, int $id): WebshopOrderInterface
    {
        return $this->find([$this->getWebshop($webshop), $id]) ?: new WebshopOrder($webshop, $id);
    }

    public function update(int|string|WebshopInterface $webshop, int $id): WebshopOrderInterface
    {
        $webshop = $this->getWebshop($webshop);
        $order = $this->findOrCreate($webshop, $id);

        $order->update();

        $this->_em->persist($order);
        $this->_em->flush();

        return $order;
    }

    private function getWebshop(int|string|WebshopInterface $idOrDomain): WebshopInterface
    {
        if ($idOrDomain instanceof WebshopInterface) {
            return $idOrDomain;
        }

        /** @var WebshopRepository $webshops */
        $webshops = $this->_em->getRepository(Webshop::class);

        if (is_string($idOrDomain)) {
            $webshop = $webshops->findByDomain($idOrDomain);
        } else {
            $webshop = $webshops->find($idOrDomain);
        }

        if ($webshop === null) {
            throw new InvalidArgumentException(
                sprintf('Nepostojeći webshop %s.', $idOrDomain)
            );
        }

        return $webshop;
    }
}