<?php
declare(strict_types=1);

namespace Catequil\Webshop\Service;

use Catequil\Webshop\Model\WebshopInterface;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Events;
use Gath\Api\EndpointPluginManager;
use Gath\Client;
use Gath\ClientInterface;
use GuzzleHttp\Psr7\Uri;
use Laminas\Hydrator\HydratorPluginManager;
use Psr\Container\ContainerInterface;

/**
 * Class WebshopEventSubscriber
 *
 * @package Catequil\Webshop\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class WebshopEventSubscriber implements EventSubscriber
{
    private ContainerInterface $container;

    /**
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function getSubscribedEvents(): array
    {
        return [
            Events::postLoad,
        ];
    }

    public function postLoad(LifecycleEventArgs $args): void
    {
        $webshop = $args->getEntity();

        if (! $webshop instanceof WebshopInterface) {
            return;
        }

        if ($gathUrl = $webshop->getGathUrl()) {
            $url = new Uri($gathUrl);
            /** @noinspection PhpUnhandledExceptionInspection */
            $endpoints = $this->container->get(EndpointPluginManager::class);
            /** @noinspection PhpUnhandledExceptionInspection */
            $hydrators = $this->container->get(HydratorPluginManager::class);
            $client = new Client($url, $endpoints, $hydrators);
            $webshop->setGathClient($client);
        }
    }


}