<?php
declare(strict_types=1);

namespace Catequil\Webshop\Service;

use Catequil\Webshop\Model\Webshop;
use Catequil\Webshop\Model\WebshopInterface;
use Catequil\Webshop\Model\WebshopOrder;
use Catequil\Webshop\Repository\WebshopOrderRepository;
use Doctrine\ORM\EntityManagerInterface;

/**
 * Class WebshopService
 *
 * @package Catequil\Webshop\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
final class WebshopService implements WebshopServiceInterface
{
    private EntityManagerInterface $em;

    /**
     * @param EntityManagerInterface $em
     */
    public function __construct(EntityManagerInterface $em)
    {
        $this->em = $em;
    }

    public function orders(): WebshopOrderRepository
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->em->getRepository(WebshopOrder::class);
    }

    public function findById(int $id): ?WebshopInterface
    {
        return $this->em->find(Webshop::class, $id);
    }

    /**
     * @param string $domain
     * @return WebshopInterface|null
     */
    public function findByDomain(string $domain): ?WebshopInterface
    {
        $repo = $this->em->getRepository(Webshop::class);
        return $repo->findOneBy(['domain' => $domain]);
    }
}